//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  fstring.cpp
//! \brief Collection of string functions.


#include <iostream>
#include <map>
#include <vector>

#include "defs.hpp"
#include "commands.hpp"
#include "func.hpp"
#include "fregistry.hpp"


//**************************************************************************************************
//*  Functions of one argument
//**************************************************************************************************

void func_cmd(string s) { frida_command( s ); } // TO CHECK: -> "eval" ? right approach ?

int func_len(string s) { return s.size(); }

//**************************************************************************************************
//*  Functions of two arguments
//**************************************************************************************************

string func_mul(string s, int n)
{
    string ret = s;
    for( int i=1; i<n; ++i )
        ret += s;
    return ret;
}

//**************************************************************************************************
//*  Registration
//**************************************************************************************************

//! Registers basic and arithmetic functions.

void fstring_initialize()
{

#define G GFuncRegistry

    // operators by precedence (which should agree with xax_yacc.ypp):
    G->register_fct_s_si ( "*", func_mul );

    // f( 1 arg )
    G->register_fct_meta ( "cmd", 1, "(s): execute s as Frida command" );
    G->register_fct_0_s  ( "cmd", func_cmd );
    G->register_fct_meta ( "len", 1, "(s): length of string s" );
    G->register_fct_i_s  ( "len", func_len );
}